from sqlalchemy.exc import IntegrityError
from werkzeug.exceptions import BadRequest, NotFound

from flaschengeist.models.user import Role, Permission
from flaschengeist.database import db
from flaschengeist import logger


def get_all():
    return Role.query.all()


def get(role_name):
    role = Role.query.filter(Role.name == role_name).one_or_none()
    if not role:
        raise NotFound


def get_permissions():
    return Permission.query.all()


def update_role(role):
    db.session.commit()


def set_permissions(role, permissions):
    for name in permissions:
        p = Permission.query.filter(Permission.name == name).one_or_none()
        if not p:
            raise BadRequest("Invalid permission name >{}<".format(name))
        role.permissions.append(p)
    db.session.commit()


def create_permissions(permissions):
    for permission in permissions:
        if Permission.query.filter(Permission.name == permission).count() > 0:
            continue
        p = Permission(name=permission)
        db.session.add(p)
        db.session.commit()


def create_role(name: str, permissions=[]):
    role = Role(name=name)
    db.session.add(role)
    set_permissions(role, permissions)
    return role


def delete(role):
    try:
        num = Role.query.filter(Role.id == role.id).delete()
    except IntegrityError:
        logger.debug("IntegrityError: Role might still be in use", exc_info=True)
        raise BadRequest("Role still in use")
    db.session.commit()
    return num == 1
